@@ -15,11 +15,12 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
@@ -31,7 +32,7 @@
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
 struct bit_desc {
-	u_int8_t mask;
+	uint8_t mask;
 	char *name;
 };
 
@@ -41,8 +42,8 @@ struct io_index {
 };
 
 struct io_reg {
-	u_int16_t	io_port_addr;	/* port for address */
-	u_int16_t	io_port_data;	/* port for address */
+	uint16_t	io_port_addr;	/* port for address */
+	uint16_t	io_port_data;	/* port for address */
 	char *		name;
 	struct io_index	index[0xff];
 };
@@ -97,6 +98,16 @@ struct io_reg graphic_regs = {
 	},
 };
 
+
+static struct bit_desc crtc_32_desc[] = {
+	{ 0x01, "Real-Time Flipping", },
+	{ 0x02, "Digital Video Port (DVP) Grammar Correction", },
+	{ 0x04, "Display End Blanking Enable", },
+	{ 0x08, "CRT SYNC Driving Selection (0: Low, 1: High)", },
+	{ 0xE0, "HSYNC Delay Number by VCLK", },
+	{ 0 },
+};
+
 struct io_reg crtc_regs = {
 	.io_port_addr = 0x3d4,
 	.io_port_data = 0x3d5,
@@ -131,9 +142,9 @@ struct io_reg crtc_regs = {
 		/* CRT Controller Extended Register */
 		[0x30] = { "Display Fetch Blocking Control", },
 		[0x31] = { "Half Line Position", },
-		[0x32] = { "Mode Control", },
+		[0x32] = { "Mode Control", crtc_32_desc, },
 		[0x33] = { "Hsync Adjuster", },
-		[0x34] = { "Starting Address Overflow", },
+		[0x34] = { "Starting Address Overflow, Bits [23:16]", },
 		[0x35] = { "Extended Overflow", },
 		[0x36] = { "Power Management Control 3", },
 		[0x37] = { "DAC Control", },
@@ -175,8 +186,8 @@ struct io_reg crtc_regs = {
 		[0x64] = { "Second Display Starting Address High", },
 		[0x65] = { "Second Display Horizontal Quadword Count", },
 		[0x66] = { "Second Display Horizontal Offset", },
-		[0x67] = { "Second Display Col Depth and Horiz Overfl", },
-		[0x68] = { "Second Display Queue Depth and Read Thresh", },
+		[0x67] = { "Second Display Color Depth and Horizontal Overflow", },
+		[0x68] = { "Second Display Queue Depth and Read Threshold", },
 		[0x69] = { "Second Display Interrupt Enable and Status", },
 		[0x6a] = { "Second Display Channel and LCD Enable", },
 		[0x6b] = { "Channel 1 and 2 Clock Mode Selection", },
@@ -192,7 +203,7 @@ struct io_reg crtc_regs = {
 		[0x75] = { "Vertical Retrace Start Shadow", },
 		[0x76] = { "Vertical Retrace End Shadow", },
 		[0x77] = { "LCD Horizontal Scaling Factor", },
-		[0x78] = { "LCD Vertical Scaling Facor", },
+		[0x78] = { "LCD Vertical Scaling Factor", },
 		[0x79] = { "LCD Scaling Control", },
 		[0x7a] = { "LCD Scaling Parameter 1", },
 		[0x7b] = { "LCD Scaling Parameter 2", },
@@ -216,7 +227,7 @@ struct io_reg crtc_regs = {
 		[0x8e] = { "LCD Power Sequence Control 3", },
 		[0x8f] = { "LCD Power Sequence Control 4", },
 		[0x90] = { "LCD Power Sequence Control 5", },
-		[0x91] = { "Software Cotnrol Power Sequence", },
+		[0x91] = { "Software Control Power Sequence", },
 		[0x92] = { "Read Threshold 2", },
 		[0x94] = { "Expire Number and Display Queue Extend", },
 		[0x95] = { "Extend Threshold Bit", },
@@ -239,8 +250,8 @@ struct io_reg crtc_regs = {
 		[0xa8] = { "Expected IGA1 Vertical Display End", },
 		[0xa9] = { "Hardware Gamma Control", },
 		[0xaa] = { "FIFO Depth + Threshold Overflow", },
-		[0xab] = { "IGA2 Inetrlace Half Line", },
-		[0xac] = { "IGA2 Inetrlace Half Line", },
+		[0xab] = { "IGA2 Interlace Half Line", },
+		[0xac] = { "IGA2 Interlace Half Line", },
 		[0xaf] = { "P-Arbiter Write Expired Number", },
 		[0xb0] = { "IGA2 Pack Circuit Request Threshold", },
 		[0xb1] = { "IGA2 Pack Circuit Request High Threshold", },
@@ -318,6 +329,16 @@ static struct bit_desc seq_19_desc[] = {
 	{ 0 },
 };
 
+
+static struct bit_desc seq_1a_desc[] = {
+	{ 0x01, "LUT Shadow Access", },
+	{ 0x04, "PCI Burst Write Wait State Select (0: 0 Wait state, 1: 1 Wait state)", },
+	{ 0x08, "Extended Mode Memory Access Enable (0: Disable, 1: Enable)", },
+	{ 0x40, "Software Reset (0: Default value, 1: Reset)", },
+	{ 0x80, "Read Cache Enable (0: Disable, 1: Enable)", },
+	{ 0 },
+};
+
 static struct bit_desc seq_1b_desc[] = {
 	{ 0x01, "Primary Display's LUT Off", },
 	{ 0x18, "Primary Display Engine VCK Gating", },
@@ -337,7 +358,7 @@ static struct bit_desc seq_1e_desc[] = {
 static struct bit_desc seq_2a_desc[] = {
 	{ 0x03, "LVDS Channel 1 Pad Control" },
 	{ 0x0c, "LVDS Channel 2 Pad Control" },
-	{ 0x40, "Sprad Spectrum Type FIFO" },
+	{ 0x40, "Spread Spectrum Type FIFO" },
 	{ 0 },
 };
 
@@ -354,7 +375,7 @@ static struct bit_desc seq_2b_desc[] = {
 static struct bit_desc seq_2d_desc[] = {
 	{ 0x03, "ECK Pll Power Control", },
 	{ 0x0c, "LCK PLL Power Control", },
-	{ 0x30, "VCK PLL Powre Control", },
+	{ 0x30, "VCK PLL Power Control", },
 	{ 0xc0, "E3_ECK_N Selection", },
 	{ 0 },
 };
@@ -400,7 +421,7 @@ static struct bit_desc seq_43_desc[] = {
 	{ 0x04, "Typical Channel 1 Arbiter Read Back Data Overwrite Flag", },
 	{ 0x08, "Typical Channel 0 Arbiter Read Back Data Overwrite Flag", },
 	{ 0x10, "IGA1 Display FIFO Underflow Flag", },
-	{ 0x20, "IGA2 Dispaly FIFO Underflow Flag", },
+	{ 0x20, "IGA2 Display FIFO Underflow Flag", },
 	{ 0x40, "Windows Media Video Enable Flag", },
 	{ 0x80, "Advance Video Enable Flag", },
 	{ 0 },
@@ -424,7 +445,7 @@ static struct bit_desc seq_59_desc[] = {
 	{ 0x04, "GFX-NM GMINT Channel 1 Dynamic Clock Enable", },
 	{ 0x08, "GFX-NM PCIC Dynamic Clock Enable", },
 	{ 0x10, "GFX-NM IGA Dynamic Clock Enable", },
-	{ 0x20, "IGA Low Thrshold Enable", },
+	{ 0x20, "IGA Low Threshold Enable", },
 	{ 0x80, "IGA1 Enable", },
 	{ 0 },
 };
@@ -477,8 +498,8 @@ struct io_reg sequencer_regs = {
 		[0x17] = { "Display FIFO Control", },
 		[0x18] = { "Display Arbiter Control 0", },
 		[0x19] = { "Power Management", seq_19_desc, },
-		[0x1a] = { "PCI Bus Control", },
-		[0x1b] = { "Power Management Control 0", seq_1b_desc,  },
+		[0x1a] = { "PCI Bus Control", seq_1a_desc, },
+		[0x1b] = { "Power Management Control 0", seq_1b_desc, },
 		[0x1c] = { "Horizontal Display Fetch Count Data", },
 		[0x1d] = { "Horizontal Display Fetch Count Control", },
 		[0x1e] = { "Power Management Control", seq_1e_desc, },
@@ -506,14 +527,14 @@ struct io_reg sequencer_regs = {
 		[0x3f] = { "Power Management Control 2", seq_3f_desc, },
 		[0x40] = { "PLL Control", seq_40_desc, },
 		[0x41] = { "Typical Arbiter Control 1", },
-		[0x42] = { "Typical Arbiter Control 1", },
+		[0x42] = { "Typical Arbiter Control 2", },
 		[0x43] = { "Graphics Bonding Option", seq_43_desc, },
-		[0x44] = { "VCK Clock Synthesizer Vallue 0", },
-		[0x45] = { "VCK Clock Synthesizer Vallue 1", },
-		[0x46] = { "VCK Clock Synthesizer Vallue 2", },
-		[0x47] = { "ECK Clock Synthesizer Vallue 0", },
-		[0x48] = { "ECK Clock Synthesizer Vallue 1", },
-		[0x49] = { "ECK Clock Synthesizer Vallue 2", },
+		[0x44] = { "VCK Clock Synthesizer Value 0", },
+		[0x45] = { "VCK Clock Synthesizer Value 1", },
+		[0x46] = { "VCK Clock Synthesizer Value 2", },
+		[0x47] = { "ECK Clock Synthesizer Value 0", },
+		[0x48] = { "ECK Clock Synthesizer Value 1", },
+		[0x49] = { "ECK Clock Synthesizer Value 2", },
 		[0x4a] = { "LDCK Clock Synthesizer Value 0", },
 		[0x4b] = { "LDCK Clock Synthesizer Value 1", },
 		[0x4c] = { "LDCK Clock Synthesizer Value 2", },
@@ -543,14 +564,14 @@ struct io_reg sequencer_regs = {
 		[0x65] = { "Power Management Control 6", },
 		[0x66] = { "GTI Control 0", },
 		[0x67] = { "GTI Control 1", },
-		[0x68] = { "GTI Control 1", },
-		[0x69] = { "GTI Control 1", },
-		[0x6a] = { "GTI Control 1", },
-		[0x6b] = { "GTI Control 1", },
-		[0x6c] = { "GTI Control 1", },
-		[0x6d] = { "GTI Control 1", },
-		[0x6e] = { "GTI Control 1", },
-		[0x6f] = { "GTI Control 1", },
+		[0x68] = { "GTI Control 2", },
+		[0x69] = { "GTI Control 3", },
+		[0x6a] = { "GTI Control 4", },
+		[0x6b] = { "GTI Control 5", },
+		[0x6c] = { "GTI Control 6", },
+		[0x6d] = { "GTI Control 7", },
+		[0x6e] = { "GTI Control 8", },
+		[0x6f] = { "GTI Control 9", },
 		[0x70] = { "GARB Control 0", },
 		[0x71] = { "Typical Arbiter Control 2", },
 		[0x72] = { "Typical Arbiter Control 3", },
@@ -563,23 +584,23 @@ struct io_reg sequencer_regs = {
 	},
 };
 
-static u_int8_t readb_idx_reg(u_int16_t port, u_int8_t index)
+static uint8_t readb_idx_reg(uint16_t port, uint8_t index)
 {
 	outb(index, port-1);
 	return inb(port);
 }
 
-static void writeb_idx_reg(u_int16_t port, u_int8_t index,
-			   u_int8_t val)
+static void writeb_idx_reg(uint16_t port, uint8_t index,
+			   uint8_t val)
 {
 	outb(index, port-1);
 	outb(val, port);
 }
 
-static void writeb_idx_mask(u_int16_t reg, u_int8_t idx, u_int8_t val,
-			    u_int8_t mask)
+static void writeb_idx_mask(uint16_t reg, uint8_t idx, uint8_t val,
+			    uint8_t mask)
 {
-	u_int8_t tmp;
+	uint8_t tmp;
 
 	tmp = readb_idx_reg(reg, idx);
 	tmp &= ~ mask;
@@ -594,31 +615,31 @@ struct io_reg *io_regs[] = {
 	&graphic_regs,
 	&crtc_regs,
 	NULL
-};	
+};
 
 struct half_mode {
-	u_int16_t total;
-	u_int16_t active;
-	u_int16_t blank_start;
-	u_int16_t blank_end;
-	u_int16_t retr_start;
-	u_int16_t retr_end;
+	uint16_t total;
+	uint16_t active;
+	uint16_t blank_start;
+	uint16_t blank_end;
+	uint16_t retr_start;
+	uint16_t retr_end;
 	int n_sync;
 };
 
 struct mode {
-	struct half_mode h;	
-	struct half_mode v;	
-	u_int32_t addr_start;
-	u_int8_t bpp;
-	u_int16_t horiz_quad_count;
-	u_int16_t horiz_offset;
+	struct half_mode h;
+	struct half_mode v;
+	uint32_t addr_start;
+	uint8_t bpp;
+	uint16_t horiz_quad_count;
+	uint16_t horiz_offset;
 };
 
 static int get_mode(struct mode *m, int secondary)
 {
-	u_int8_t val;
-	
+	uint8_t val;
+
 	memset(m, 0, sizeof(*m));
 
 	if (!secondary) {
@@ -781,7 +802,7 @@ static int get_mode(struct mode *m, int secondary)
 static void dump_scaling(void)
 {
 	u_int32_t h_scaling, v_scaling;
-	u_int8_t val;
+	uint8_t val;
 
 	val = readb_idx_reg(0x3d5, 0x79);
 	if (val & 0x01) {
@@ -807,11 +828,11 @@ static void dump_scaling(void)
 
 static void dump_registers(struct io_reg *ior)
 {
-	u_int8_t idx;
+	uint8_t idx;
 
-	printf("%s register dump:\n", ior->name);
+	printf("%s register dump (IO Port address: 0x%03x): \n", ior->name, ior->io_port_addr);
 	for (idx = 0; idx < 0xff; idx++) {
-		u_int8_t val;
+		uint8_t val;
 		struct bit_desc *desc = ior->index[idx].bit_desc;
 
 		if (!ior->index[idx].name)
@@ -819,7 +840,7 @@ static void dump_registers(struct io_reg *ior)
 
 		outb(idx, ior->io_port_addr);
 		val = inb(ior->io_port_data);
-		printf("    0x%02x = 0x%02x (%s)\n", idx, val,	
+		printf("   %03x.%02x = 0x%02x (%s)\n", ior->io_port_data, idx, val,	
 			ior->index[idx].name);
 		
 		if (!desc) 
@@ -844,8 +865,8 @@ enum pll {
 
 static void get_vck_clock(enum pll pll, unsigned int f_ref_khz)
 {
-	u_int8_t reg_ofs = 0;
-	u_int8_t val;
+	uint8_t reg_ofs = 0;
+	uint8_t val;
 	unsigned int dm, dtz, dr, dn;
 	unsigned long f_vco, f_out;
 	char *name;
@@ -878,7 +899,7 @@ static void get_vck_clock(enum pll pll, unsigned int f_ref_khz)
 	dtz |= (val & 0x1) << 1;
 	dn = val >> 1;
 
-	printf("%s PLL: dm=%d, dtx=%d, dr=%d, dn=%d ", name, dm, dtz, dr, dn);
+	printf("%s PLL: dm=%u, dtx=%u, dr=%u, dn=%u ", name, dm, dtz, dr, dn);
 
 	f_vco = f_ref_khz * (dm + 2) / (dn + 2);
 	if (dr)
@@ -898,7 +919,7 @@ struct gpio_state {
 
 static int get_gpio_state(struct gpio_state *s)
 {
-	u_int8_t val;
+	uint8_t val;
 
 	memset(s, 0, sizeof(*s));
 
@@ -938,7 +959,7 @@ static int get_gpio_state(struct gpio_state *s)
 
 static void dump_gpio_state(const char *pfx, const struct gpio_state *gs)
 {
-	int i;
+	unsigned int i;
 
 	for (i = 2; i < 6; i++) {
 		printf("%sGPIO %u: function=", pfx, i);
@@ -978,10 +999,10 @@ static void dump_mode(const char *pfx, struct mode *m)
 
 static void dump_sl(const char *pfx)
 {
-	u_int8_t val;
+	uint8_t val;
 	unsigned int sl_size_mb;
 	unsigned long rtsf_in_sl_addr;
-	u_int64_t sl_in_mem_addr;
+	uint64_t sl_in_mem_addr, temp;
 
 	val = readb_idx_reg(0x3c5, 0x68);
 	switch (val) {
@@ -1021,7 +1042,8 @@ static void dump_sl(const char *pfx)
 
 	sl_in_mem_addr = readb_idx_reg(0x3c5, 0x6d) << 21;
 	sl_in_mem_addr |= readb_idx_reg(0x3c5, 0x6d) << 29;
-	sl_in_mem_addr |= (readb_idx_reg(0x3c5, 0x6d) & 0x7f) << 37;
+	temp = (readb_idx_reg(0x3c5, 0x6d) & 0x7f);
+	sl_in_mem_addr |= (temp << 37);
 
 	printf("%sSL in System memory: 0x%llx, RTSF in SL: 0x%lx\n",
 		pfx, sl_in_mem_addr, rtsf_in_sl_addr);
@@ -1029,7 +1051,7 @@ static void dump_sl(const char *pfx)
 
 static int dump_lvds(void)
 {
-	u_int8_t val;
+	uint8_t val;
 	char *mode;
 
 	writeb_idx_mask(0x3c5, 0x5a, 0x01, 0x01);
@@ -1067,7 +1089,7 @@ static int dump_lvds(void)
 		val & 1 ? "OpenLDI":"SPWG", val & 0x40 ? "Down" : "Up");
 	
 }
-static int parse_ioreg(u_int16_t *reg, u_int8_t *index, char *str)
+static int parse_ioreg(uint16_t *reg, uint8_t *index, char *str)
 {
 	char *dot;
 	char buf[255];
@@ -1110,8 +1132,8 @@ static void usage(void)
 	printf("-d | --dump  : Dump all registers.\n");
 	printf("-p | --pll   : Display PLL.\n");
 	printf("-m | --mode  : Display modes.\n");
-	printf("-r | --read  : Read register.\n");
-	printf("-w | --write : Write register.\n");
+	printf("-r | --read  : Read register. Example : $0 -r 3d5.17\n");
+	printf("-w | --write : Write register. Example : $0 -w 3d5.17 0xa3\n");
 	printf("-g | --gpio  : Display GPIO state.\n");
 }
 
@@ -1140,8 +1162,8 @@ int main(int argc, char **argv)
 
 	while (1) {
 		int c;
-		u_int16_t reg;
-		u_int8_t index;
+		uint16_t reg;
+		uint8_t index;
 		unsigned long val;
 		static struct option long_options[] = {
 			{ "help", 0, 0, 'h' },
